/************************************************************************/
/*                                                                      */
/*  AMD Embedded Flash Memory Card Evaluation Kit                       */
/*  File name: EMBED.C                                                  */
/*  Revision:  2.0  11/20/96                                            */
/*                                                                      */
/* Copyright (c) 1995 ADVANCED MICRO DEVICES, INC. All Rights Reserved. */
/* This software is unpublished and contains the trade secrets and      */
/* confidential proprietary information of AMD. Unless otherwise        */
/* provided in the Software Agreement associated herewith, it is        */
/* licensed in confidence "AS IS" and is not to be reproduced in whole  */
/* or part by any means except for backup. Use, duplication, or         */
/* disclosure by the Government is subject to the restrictions in       */
/* paragraph (b) (3) (B) of the Rights in Technical Data and Computer   */
/* Software clause in DFAR 52.227-7013 (a) (Oct 1988).                  */
/* Software owned by                                                    */
/* Advanced Micro Devices, Inc.,                                        */
/* One AMD Place,                                                       */
/* P.O. Box 3453                                                        */
/* Sunnyvale, CA 94088-3453.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Revision History                                                     */
/*                                                                      */
/* Rev.  Date        By    Comments                                     */
/* 1.2   2/12/96     KT    Initial Release to NVD                       */
/* 1.21  3/21/96     KT    Added selection 'x' for device selection     */
/* 1.22  5/21/96     KT    Fixed max address checking for commands 'p'  */
/*                         and 'm'.  Command 'm' is now visible.        */
/* 1.4B  6/25/96     KT    Socket related files socket.c,socket.h in C. */
/* 1.5B  8/15/96     KT    Change deadman timer for dq7_poll to 9 secs. */
/* 1.6B  9/25/96     KD    Added support for F016 chip cards            */
/* 1.7B  9/25/96     KD    Added support for Miniature Cards, LV008     */
/* 1.8B  10/8/96     KD    Changed use of socket 0 to using socket 1.   */
/*       10/10/96    KD    Added option for either socket 0 or 1.       */
/* 1.81  10/10/96    KD    Changed algorithms to that for F016C         */
/*                         Added EMM warning message on first screen    */
/*                         Displayed socket & card in use.              */
/*                         Added x option in autoselect screen.         */
/* 1.8B  10/11/96    KD    Updated 1.81 into 1.8B w/o F016C routines    */
/* 1.8B  10/15/96    KD    Changed lv008 support to lv080 support.      */
/* 1.8B  10/18/96    KD    Added in automatic RESET for AUTOSELECT.     */
/* 1.8B  11/01/96    KD    Fixed boundary errors in read/erase.         */
/* 1.8B  11/20/96    KD    Fixed sector erasing of last byte in sector  */
/* 2.0   11/20/96    KD    Cleaned up some menus of 1.8B                */
/*                         Doesn't allow read past end of card.         */
/*       02/03/97    KD    Added in address exceeds limits on erase.    */
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h> 
#include <conio.h>
#include <ctype.h>
#include <time.h>
#include <bios.h>
#include "embed.h"

unsigned char slotoff = 0x00;   /* slot 0 default */
/********************************************************************/
/*      main()                                                      */
/*                                                                  */
/*      purpose: Entry point of Flash evaluation kit program source */
/*               code. This initializes the hardware and program.   */
/*               This would be called by the menu shell.            */
/*                                                                  */
/********************************************************************/
/*      parameters:     none                                        */
/******************************************************************/
/*  return value: none                                            */
/******************************************************************/
void main()
{

	int  end_prog_L=0, err_val_L=0;
	char selection_L, selection2_L;
	double tused;
	time_t tstart;
	time_t tstop;
	char c;
	int socketnum;

/* Even before menu is displayed, need to do system initialization
first.
1. Initialize SCM Adapter.
2. Set up Adapter for polling.
3. Power supply is auto configured.
4. Acquire memory windows.
5. Memory Window management

*/      
		system ("cls");
		printf("\n\n\n\n\n\t\t AMD Flash Memory Card Evaluation Kit \n");
		printf("\n\t\t\t (C) Copyright AMD 1996\n");
		printf("\t\t\t     Version 2.0 \n");
		printf("\n\n\n\t MAKE SURE THAT THE FOLLOWING LINE");
		printf(" IS IN YOUR CONFIG.SYS FILE!!\n");
		printf("\n\t\t DEVICE = EMM386.EXE NOEMS X-D000-DFFF \n\n\n");
		printf("\n\t Please note that Embed is intended to");
		printf(" be used in a \n\t DOS/Win 3.1 Environment and not in");
		printf(" a Windows95 Environment.\n");
		printf("\n\n\t\t [0] Socket 0\n");
		printf("\t\t [1] Socket 1\n");
		printf("\n\t\t Select socket in use: ");       
			
		selection_L = getche();
		printf("\n");
	
		switch (selection_L) {
			/* Slot 0 */
			case '0':
				slotoff = 0x00;
				socketnum = 0;
				break;
			/* Slot 1 */
			case '1':
				slotoff = 0x40;
				socketnum = 1;
				break;
			default :
				slotoff = 0x00;
				socketnum = 0;
				break;
		}/* switch slot selection */

	find_adapter();
	/*      0x604 - 100ns access time
		0x603 - 150ns access time
		0x602 - 200ns access time
		0x601 - 250ns access time */

	/* Initializations. */
	in_data_L=system_base_address;
	in_data_L_W=system_base_address;
	cardoffset=0;
	cardaddress=0;

	/* display the menu */
	do 
	{
		set_window(0xd0,0x603);
		memctrl=CM_WRITE;
		err_val_L=set_page(cardoffset_DI,memctrl);

		system ("cls");
		printf("\n\n\n\n\n\t\t AMD Flash Memory Card Evaluation Kit \n");
		printf("\n\t\t (C) Copyright AMD 1996\n");
		printf("\t\t Version	2.0 \n");
		printf("\n\t\t\tMain Menu\n");
		printf("\t\t\t---------\n");
		printf("\t\t[F]lash Card select	(Card selected: %s in slot %i)\n", this_dev_GSP->devname_GS, socketnum);                       
		printf("\t\t[R]ead memory\n");
		printf("\t\t[M]emory to file\n");
		printf("\t\t[W]ord - program single word\n");
		printf("\t\t[B]yte - program single byte\n");
		printf("\t\t[P]rogram user file\n");
		printf("\t\t[S]ector erase\n");
		printf("\t\t[C]ard erase\n");
/*              printf("\t\t[I]nit PCIC\n\n");*/
		printf("\t\t[X]Erase and suspend\n");
		printf("\t\t[Y]Resume Erase\n");
		printf("\t\t[A]ttribute Memory Access\n\n");
		printf("\t\t[E]xit to DOS temporarily\n");              
		printf("\t\t[Q]uit program\n\n");
		printf("Please enter Letter of needed selection: ");
      selection_L = getche();
		printf("\n");
	
		switch (selection_L) {
			/* Initialize PCIC              */
			case 'i':
			case 'I':       
				if (err_val_L=0)
					err_val_L=set_page(0,6);                                                    if (err_val_L != 0) 
					error_handler(err_val_L);
				printf("\n");
				err_val_L=0;
				break;

			/* Select the device to use */
			case 'd':
			case 'D':
			case 'f':
			case 'F':
				if (err_val_L != 0) 
					error_handler(err_val_L);
				cmd_seq(RESET);         /* resets before autoselect */
				err_val_L = sel_device();
				max_addr_L = this_dev_GSP->kb_size_GS;
				if (err_val_L != 0) {
					error_handler(err_val_L);
					this_dev_GSP = &amd_devices_GA[0];
				}
				else {
					printf("Hit Enter to continue\n");
					getche();
				}
				printf("\n\n");
				break;

			/* Test Address Check */
			case 'z':
			case 'Z':
				printf("\nPlease enter the appropriate sector address to erase: ");
				scanf("%Lx%n", &cardaddress);
				printf("cardaddress=%10lx\n",cardaddress);
				address_check();
				printf("After Check:\n");
				printf("cardaddress=%10lx\n",cardaddress);
				printf("system_address=%10lx\n",system_address);
				printf("cardoffset=%10lx\n",cardoffset);
				printf("cardoffset_di=%6x\n",cardoffset_DI);
				printf("window_size=%10lx\n",window_size);
				printf("\n");
				fflush(stdin);
				getche();
				break;

			/* Read the device selected */
			case 'r':
			case 'R':       
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else 
					err_val_L=read_bin();
				if (err_val_L != 0) 
					error_handler(err_val_L);
				printf("\n");
				err_val_L=0;
				break;

			/* Program a single byte on the device */
			case 'b':
			case 'B':       
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					printf("Please enter the appropriate address to program: ");
					scanf("%Lx", &cardaddress);
					if (cardaddress >= max_addr_L) 
						err_val_L = ADDRESS_EXCEEDS_LIMITS;
					else {
						printf("Please enter the data byte to program: ");
						scanf("%Lx", &write_data_B);
						err_val_L = program_byte();
					}
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nProgram Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Program a single word on the device */
			case 'w':
			case 'W':       
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					fflush(stdin);
					printf("Please enter the appropriate address to program: ");
					scanf("%Lx", &cardaddress);
					if (cardaddress >= max_addr_L) 
						err_val_L = ADDRESS_EXCEEDS_LIMITS;
					else {
						fflush(stdin);
						printf("Please enter the data word to program: ");
						scanf("%Lx", &write_data);
						if ((cardaddress & 0x1) !=0) 
						{
							odd_offset=1;
							write_data_B=(char) (write_data&0x00ff);
							err_val_L = program_byte();
							if (err_val_L == 0)
							{
								odd_offset=0;
								write_data_B=(char) (write_data>>8);
								cardaddress++;
								err_val_L = program_byte();
							}
						}
						else
							err_val_L = program_word();
					}
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nProgram Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Program a device */
			case 'p':
			case 'P':       
				fflush(stdin);
				if (no_device_G)
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					printf("Please enter file name:");
					gets(filename);
					printf("File is %s\n",filename);
					printf("Please enter the appropriate address to program: ");
					scanf("%Lx", &cardaddress);
					if (cardaddress >= max_addr_L) 
						err_val_L = ADDRESS_EXCEEDS_LIMITS;
					else 
						err_val_L = program_cm_file();
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nHit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				fcloseall();
				break;

			/* Read data into a file */
			case 'm':
			case 'M':       
				fflush(stdin);
				if (no_device_G)
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					printf("Please enter file name:");
					gets(filename);
					printf("File is %s\n",filename);
					printf("Please enter the appropriate address to read: ");
					scanf("%Lx", &cardaddress);
					printf("Please enter the number of bytes to read: ");
					scanf("%Lx", &numbytes);
					if ((cardaddress>=max_addr_L)||((cardaddress+numbytes)>max_addr_L)) 
						err_val_L = ADDRESS_EXCEEDS_LIMITS;
					else 
						err_val_L = read_cm_file();
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nHit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				fcloseall();
				break;

			/* Perform a Card Erase */
			case 'c':
			case 'C':       
				fflush(stdin);
				printf("\nPlease confirm[Y/N]: ");
				scanf("%s", &selection_L);
				printf("\n");   
				if (!((selection_L=='y') || (selection_L=='Y')))
					break;
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {  
					printf("This may take a few minutes.\n");       
					time(&tstart);
					err_val_L=card_erase();
					time(&tstop);
					tused=difftime(tstop,tstart);
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nTime taken for erase = %f seconds.\n",tused);
					printf("\nCard Erase Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Perform a Sector Erase */
			case 's':
			case 'S':       
				fflush(stdin);
				printf("\nPlease confirm[Y/N]: ");
				scanf("%s", &selection_L);
				printf("\n");   
				if (!((selection_L=='y') || (selection_L=='Y')))
					break;
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					printf("Please enter the appropriate sector address to erase: ");
					scanf("%Lx", &cardaddress);
					printf("cardaddress=%10lx\n",cardaddress);
					time(&tstart);
					err_val_L=sector_erase();
					time(&tstop);
					tused=difftime(tstop,tstart);
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nTime taken for erase = %f seconds.\n",tused);
					printf("\nSector Erase Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Perform a Sector Erase Suspend*/
			case 'x':
			case 'X':       
				fflush(stdin);
				printf("\nPlease confirm[Y/N]: ");
				scanf("%s", &selection_L);
				printf("\n");   
				if (!((selection_L=='y') || (selection_L=='Y')))
					break;
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					fflush(stdin);
					printf("Please enter the appropriate sector address to erase: ");
					scanf("%Lx", &cardaddress);
					printf("cardaddress=%10lx\n",cardaddress);
					err_val_L=sector_erase_sus();
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nSector Erase-Suspend Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Perform a Sector Erase Resume*/
			case 'y':
			case 'Y':       
				fflush(stdin);
				printf("\nPlease confirm[Y/N]: ");
				scanf("%s", &selection_L);
				printf("\n");   
				if (!((selection_L=='y') || (selection_L=='Y')))
					break;
				if (no_device_G) 
					err_val_L = SELECT_DEVICE_FIRST;
				else {
					printf("Please enter the appropriate sector address to resume: ");
					scanf("%Lx", &cardaddress);
					printf("cardaddress=%10lx\n",cardaddress);
					err_val_L=sector_erase_res();
				}
				if (err_val_L != 0) 
					error_handler(err_val_L);
				else {
					printf("\nSector Erase-Resume Successful. Hit Return to Continue.");
					getche();
				}                                       
				printf("\n");
				err_val_L=0;
				break;

			/* Exit this program */
			case 'q':
			case 'Q':       
				end_prog_L=1;
				err_val_L=0;
				restore_regs();
				break;

			/*Exit to DOS temporarily */
			case 'e':
			case 'E':
				printf("Type EXIT to continue");
				system("COMMAND");
				break;

			/* Enter the Attribute Memory Operations menu */
			case 'a':
			case 'A':   
				if (no_device_G) {
					err_val_L = SELECT_DEVICE_FIRST;
					error_handler(err_val_L);
				}
				else {
					cardaddress=0;
					memctrl=AM_WRITE;
					address_check();
					do {
/*                                              err_val_L = set_page(0,7);*/
						system ("cls");
						printf("\n\t\t\tAttribute Memory Menu\n");
						printf("\t\t\t-----------------------\n");                  
						printf("\t\t[D]isplay Attribute memory\n");
						printf("\t\t[M]odify Attribute Memory\n");
						printf("\t\t[W]rite file to Attribute memory\n");
						printf("\t\t[F]ile image to Attribute memory\n");
						printf("\t\t[R]ead Attribute memory to file\n");
						printf("\t\t[E]xit to DOS temporarily\n");              
						printf("\t\t[B]ack to previous menu\n");                
						printf("Please enter Letter of needed selection: ");
						selection2_L = getche();
						printf("\n");                                                                                           

						switch(selection2_L) 
						{
						/* Display Attribute Memory */
							case 'd':
							case 'D':
								err_val_L = set_page(0,7);
								if (err_val_L != 0) 
									error_handler(err_val_L);
								else 
									err_val_L=read_bin();     
								if (err_val_L != 0) 
									error_handler(err_val_L);
								printf("\n");
								err_val_L=0;
								break;
						
						/* Modify Attribute Memory */
							case 'm':
							case 'M':
								set_page(0,3);
								err_val_L=write_bin();
								if (err_val_L != 0) 
									error_handler(err_val_L);
								set_page(0,7);
								printf("\n");
								err_val_L=0;
								break;

						/* Program a device */
							case 'w':
							case 'W':       
						/* must ask user for file name here and
							any parsing needed */
								fflush(stdin);
								printf("\nEnter CIS File name: ");
								gets(filename);
								set_page(0,3);
								err_val_L=program_attr_file();
								if (err_val_L != 0) 
									error_handler(err_val_L);
								err_val_L = set_page(0,7);
								printf("\n");
								fcloseall();
								err_val_L=0;
								break;

							case 'f':
							case 'F':       
						/* must ask user for file name here and
							any parsing needed */
								fflush(stdin);
								printf("\nEnter CIS File name: ");
								gets(filename);
								set_page(0,3);
								err_val_L=program_attr_file_image();
								if (err_val_L != 0) 
									error_handler(err_val_L);
								err_val_L = set_page(0,7);
								printf("\n");
								fcloseall();
								err_val_L=0;
								break;

						/* Program a device */
							case 'r':
							case 'R':       
						/* must ask user for file name here and
							any parsing needed */
								fflush(stdin);
								printf("\nEnter CIS File name: ");
								gets(filename);
								printf("File is %s\n",filename);
								set_page(0,3);
								err_val_L=read_attr_file_image();
								if (err_val_L != 0) 
									error_handler(err_val_L);
								err_val_L = set_page(0,7);
								printf("\n");
								fcloseall();
								err_val_L=0;
								break;

						/*Exit to DOS temporarily */
							case 'e':
							case 'E':
								printf("Type EXIT to continue");
								system("COMMAND");
								break;

						/* Return to the previous menu */
							case 'b':
							case 'B':
								default : selection2_L = 'b';
								break;

						} /* switch ( selection2_L ) */
					} while (selection2_L != 'b' && selection2_L != 'B');

			 }/* End Attribute Memory Operations */
			default:
		break;
		} /* switch (selection_L ) */
	} while (!end_prog_L);

} /* main */

/******************************************************************/
/* sel_device()                                                   */
/*                                                                */
/* purpose: This function is selected when the user has         */
/*          selected the device and type of package to use.     */
/*          The device is automatically verified by performing  */
/*          the Auto-select functions, manuf_code, to check     */
/*          the Manufacturer's ID, and device_code, to check    */
/*          for the correct device in the socket.                                       */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                      */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int sel_device(void)
{
	char selection_L;
	
	flerror_code_G = 0;
	system ("cls");
	printf("\n");
	printf("\n\tDevice Selection Menu");
	printf("\n\t---------------------\n");     
	printf("\t[1]	AMC001CFLKA  \t\t[2]	AMC002CFLKA\n");
	printf("\t[3]	AMC004CFLKA  \t\t[4]	AMC010CFLKA\n");
	printf("\t[5]	AMC004DFLKA  \t\t[6]	AMC008DFLKA\n");
	printf("\t[7]	AMC020DFLKA  \t\t[a]	4MB Card (F016)\n");
	printf("\t[b]	AmMC002A      \t\t[c]	AmMC004A\n");
	printf("\t[d]	AmMC008A      \t\t[e]	AmMCL002A\n");
	printf("\t[f]	AmMCL004A     \t\t[x]	Other AMD/Fujitsu Flash Card\n");
	printf("\t[8]	FUC032DFLKA OLD(04AD)\n");
	printf("\t[9]	FUC032DFLKA NEW(043D)\n");
	printf("\n");
	printf("\t\t\t[0]	 Exit\n");
	printf("Enter the Flash card to use: ");    
	selection_L = getche();
	printf("\n");
	
	switch ( selection_L ) {
		case '0' :
			return(0);              /* maintain old parameters(if any) */

		case '1' :
			this_dev_GSP = &amd_devices_GA[1];
			break;
		
		case '2' :
			this_dev_GSP = &amd_devices_GA[2];
			break;

		case '3' :
			this_dev_GSP = &amd_devices_GA[3];
			break;

		case '4' :
			this_dev_GSP = &amd_devices_GA[4];
			break;

		case '5' :
			this_dev_GSP = &amd_devices_GA[5];
			break;

		case '6' :
			this_dev_GSP = &amd_devices_GA[6];
			break;

		case '7' :
			this_dev_GSP = &amd_devices_GA[7];
			break;

		case '8' :
			this_dev_GSP = &amd_devices_GA[8];
			break;

		case '9' :
			this_dev_GSP = &amd_devices_GA[9];
			break;

		case 'a' :
			this_dev_GSP = &amd_devices_GA[11];
			break;

		case 'b' :
			this_dev_GSP = &amd_devices_GA[12];
			break;

		case 'c' :
			this_dev_GSP = &amd_devices_GA[13];
			break;

		case 'd' :
			this_dev_GSP = &amd_devices_GA[14];
			break;

		case 'e' :
			this_dev_GSP = &amd_devices_GA[15];
			break;

		case 'f' :
			this_dev_GSP = &amd_devices_GA[16];
			break;


		case 'x' :
		case 'X' :
			this_dev_GSP = &amd_devices_GA[10];
			no_device_G=0;  /* reset the flag */
			printf("\nNo device was selected. AMC064DFLKA used.\n");
			return(0);

		default :
			printf("\nNo device was selected. Hit any key to Return to menu.");
			getche();
			return(0);

	} /* switch */

	/* now check to see if it is in the socket */
	if (selection_L != '0') {
		autoselect_seq();             /* do auto_select cmd     */
		if ((m_code_G == 0x0101)||(m_code_G==0x0404)) { // AMD device
			if (d_code_G == this_dev_GSP->device_id_GS) {
				/* print - ok AMD device selected is correct */
				printf("\nMan. Code= %4x   Dev. Code= %4x\n", m_code_G,d_code_G);
				printf("Correct card selected\n");
				no_device_G=0;  /* reset the flag */
				return(0);              /* successful: flerror_code = 0 */
			}
			else 
			{
			// print - uh oh not CORRECT Device 
				flerror_code_G = DEVICE_IS_NOT_CORRECT;
				no_device_G    = 1;
			}

		//Not correct device
		}
		else 
		{
		// print - uh oh not CORRECT Device 
			flerror_code_G = DEVICE_IS_NOT_CORRECT;
			no_device_G    = 1;
		}

	}
	printf("\nMan. Code= %4x   Dev. Code= %4x", m_code_G,d_code_G);
	printf("\n");                   
	return( flerror_code_G );
} /* sel_device */

/******************************************************************/
/*      error_handler()                                                */
/*                                                                */
/*      purpose:        Processes all error codes by printing out messages      */
/*              to the user.                                      */
/*                                                                */
/******************************************************************/
/*      parameters:     int err_val = error code number (0-9 not used)     */
/******************************************************************/
/*  return value:   none                                          */
/******************************************************************/
void error_handler( int err_val_L )
{
	switch(err_val_L) {
		case NO_FILE_NAME_ENTERED:
			printf("\n(%d)No file name entered!\n", err_val_L);
			printf("Usage: ihex <file, without extension>\n");
			break;
		case INPUT_FILENAME_MISSING :
			printf("\n(%d)Input filename missing!", err_val_L);
			break;
		case FILENAME_PROBLEM :
			printf("\n(%d)File name problem: invalid filename, missing file", err_val_L);
			break;
		case DATACOUNT_ERROR_1 :
			printf("\n(%d)Error: data count exceeded 512 characters", err_val_L);
			break;
		case DQ7_PROGRAM_FAILURE :
			printf("\n(%d)DQ7-Programming failure.", err_val_L);
			break;
		case DQ7_FAIL_AFTER_DQ5_1 :
			printf("\n(%d)DQ7-Programming failure after DQ5=1.", err_val_L);
			break;
		case DQ7_FAIL_AFTER_DQ5_2 :
			printf("\n(%d)DQ7-Programming failure after DQ5=1.", err_val_L);
			break;
		case DQ7_ERASE_FAILURE :
			printf("\n(%d)DQ7-Erase failure. ", err_val_L);
			break;
		case DQ7_ERASE_FAILURE_DQ5 :
			printf("\n(%d)DQ7-Erase failure after DQ5=1. ", err_val_L);
			break;
		case ADDRESS_EXCEEDS_LIMITS :
			printf("\n(%d)Error: address exceeds device limits.", err_val_L);
			break;
		case UNKNOWN_BREAK_READHEX :
			printf("\n(%d)Error: undetermined break in read_hex().", err_val_L);
			break;
		case DEVICE_IS_NOT_CORRECT :
			printf("\n(%d)Error! The device in this socket is a not %s.", err_val_L, this_dev_GSP->devname_GS);
			printf("\nPlease make another selection.");
			break;
      case SELECT_DEVICE_FIRST :
		printf("\n(%d)Error! You must select a device first in order to use this function.", err_val_L);
	break;
	case DEVICE_SIZE_EXCEEDED :
		printf("\n(%d)Aborting programming...The last address of file to program exceeds this ", err_val_L);
		printf("\ndevice's maximum limit. Please select a larger device or a smaller file to program.");
		break;
	case DQ5_ON_NO_POLL :
		printf("\n(%d)Error! Operation failed: DQ5=1 in no_dq_poll().\n", err_val_L);
		break;
		case ERASE_SUSPEND_NOT_SUPP:
			printf("\n(%d)Error! Erase suspend not available for %s.", err_val_L, this_dev_GSP->devname_GS);
			break;
		case LOCATION_NOT_ERASED:
			printf("\n(%d)Error! Write location has not been erased. %s.", err_val_L, this_dev_GSP->devname_GS);
			break;
	}
	flerror_code_G = 0;     //re-initialize it
	getche();       
} /* error_handler */

/******************************************************************/
/*      program_attr_file()                                            */
/*                                                                */
/*      purpose:        This routine reads the file and writes to the                   */
/*          Attribute memory.                                     */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                  */
/*                               = anything : unsuccessful         */
/*******************************************************************/    
/*******************************************************************/    
int program_attr_file( void )
{
	FILE *input;
	unsigned char selection_L;
	unsigned char in_byte;
	int deviceType = this_dev_GSP->device_no_GS;
	char temp[16];
	int error_write=0;

	flerror_code_G = 0;
	address_G = system_base_address;
	if (!(input=fopen(filename, "r"))) {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		printf("Writing Attribute memory\r");    
		fscanf(input,"%c", &selection_L);
		in_data_L = address_G;
		*in_data_L = selection_L;
		_bios_timeofday(_TIME_GETCLOCK, &oldcount);
		newcount=oldcount;
		while ((newcount-oldcount) < 1) {
			_bios_timeofday(_TIME_GETCLOCK, &newcount);
		}
		address_G++;
		address_G++;
	}                                                                               
	while (address_G <=(system_base_address+255));
	printf("\n");    
	fclose(input);

/* Verify data written */
	address_G = system_base_address;
	if (!(input=fopen(filename, "r"))) {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		fscanf(input,"%c", &selection_L);
		in_data_L = address_G;
		in_byte=*in_data_L;
		if (in_byte != selection_L) {
			error_write=1;
			printf("\n\tCIS Write Failed. Hit any key to continue.");
			selection_L = getche();
			printf("\n");                                   
		}
		address_G++;
		address_G++;
	}       while ((address_G <=(system_base_address+255)) && (error_write==0));

	if (error_write==0) {   
		printf("\n\tCIS Write passed. Hit any key to continue.");
		selection_L = getche();
		printf("\n");                   
	}
	fclose(input);
	return( flerror_code_G );
} /* program_attr_file_bin */

/******************************************************************/
/*      program_attr_file_image()                                      */
/*                                                                */
/*      purpose:        This routine reads the file and writes to the                   */
/*          Attribute memory.                                     */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
/******************************************************************/    
int program_attr_file_image( void )
{
	FILE *input;
	unsigned char selection_L;
	unsigned char in_byte;
	int error_write;
	flerror_code_G = 0;
	error_write=0;  
	address_G = system_base_address;
	
	if (!(input=fopen(filename, "r"))) {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		printf("Writing Attribute memory\r");
		fscanf(input,"%c", &selection_L);
		in_data_L = address_G;
		*in_data_L = selection_L;
		_bios_timeofday(_TIME_GETCLOCK, &oldcount);
		newcount=oldcount;
		while ((newcount-oldcount) < 1) {
			_bios_timeofday(_TIME_GETCLOCK, &newcount);
		}
		address_G++;
	}                                                                               
	while (address_G <=(system_base_address+255));
	printf("\n");
	fclose(input);

/* Verify data written */
/*      rewind(input);*/
	address_G = system_base_address;
	if (!(input=fopen(filename, "r"))) {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		fscanf(input,"%c", &selection_L);
		in_data_L = address_G;
		in_byte=*in_data_L;
		if (in_byte != selection_L) {
			error_write=1;
			printf("\n\tCIS Write Failed. Hit any key to continue.");
			selection_L = getche();
			printf("\n");                                   
		}
		address_G++;
	}       while ((address_G <=(system_base_address+255)) && (error_write==0));

	if (error_write==0) {   
		printf("\n\tCIS Write passed. Hit any key to continue.");
		selection_L = getche();
		printf("\n");                   
	}
	fclose(input);
	return( flerror_code_G );
} /* program_attr_file_image */

/******************************************************************/
/* read_attr_file_image()                                               */
/*                                                                */
/*      purpose:        This routine reads the device and outputs to the        */
/*              screen or to a file (in binary mode).             */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int read_attr_file_image( void )
{
	FILE *input;
	unsigned char selection_L;
	unsigned char in_byte;
	int error_write;
	flerror_code_G = 0;
	error_write=0;  
	address_G = system_base_address;
	
	if (!(input=fopen(filename, "wb")))     {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		in_data_L = address_G;
		selection_L=*in_data_L;
		printf("Reading Attribute memory\r");
		fputc(selection_L,input);
		address_G++;
	}                                                                               
	while (address_G <=(system_base_address+255));
	printf("\n");
	fclose(input);
/* Verify data written */
/*      rewind(input);*/
	address_G = system_base_address;
	if (!(input=fopen(filename, "r"))) {
		printf("Error opening file.");
		printf("\nHit any key to continue");
		getche();
		return(flerror_code_G);
	}       
	do      {
		fscanf(input,"%c", &selection_L);
		in_data_L = address_G;
		in_byte=*in_data_L;
		if (in_byte != selection_L) {
			error_write=1;
			printf("\n\tCIS Read Failed. Hit any key to continue.");
			selection_L = getche();
			printf("\n");                                   
		}
		address_G++;
	}       while ((address_G <=(system_base_address+255)) && (error_write==0));

	if (error_write==0) {   
		printf("\n\tCIS Read to file passed. Hit any key to continue.");
		selection_L = getche();
		printf("\n");                   
	}
	fclose(input);
	return( flerror_code_G );
} /* read_attr_file_image */

/******************************************************************/
/*      write_bin()                                                    */
/*                                                                */
/*      purpose:        This routine writes to the device Attribute memory.   */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int write_bin( void )
{
	char selection_L;
	int i_L;
	int j_L;
	int deviceType = this_dev_GSP->device_no_GS;
	char temp[16];
	flerror_code_G = 0;
	
	fflush(stdin);
	printf("\nWriting %s", this_dev_GSP->devname_GS);
	printf("\nEnter hexadecimal starting address: ");
	scanf("%Lx", &cardaddress);
	printf("\nEnter hexadecimal value to be written: ");
	scanf("%x", &selection_L);
	printf("\n");   

	if (cardaddress >= max_addr_L) {
		flerror_code_G=ADDRESS_EXCEEDS_LIMITS;
		return(flerror_code_G);
	}               
	flerror_code_G=address_check();
	in_data_L = system_address;
	*in_data_L = selection_L;
	/* send data to user screen, 256 bytes at a time */
	if (flerror_code_G==0){
	do {
		for (j_L=0; j_L<16; j_L++) {
			/* For each of the 16 rows */
			ultoa(cardaddress, string_GA, 16);
			printf("\n%8s: ", string_GA);
			/* Get the 16 bytes of the row */
			for ( i_L=0; i_L<16; i_L++ ) {
				in_data_L = system_address;
				cardaddress++;
				flerror_code_G=address_check();  
				printf("%2.2x ", *in_data_L);                                   
				temp[i_L] = *in_data_L;
				if (temp[i_L] >= 0x07 && temp[i_L] <= 0x0d)
					temp[i_L] = 0x2e;
			} /* for each of the 16 bytes in a row */
		
	 /* Filter the output */
			for ( i_L = 0; i_L < 16; i_L++ ) {
				if ( temp[i_L] > 127) 
				/* filter system is on & is unprintable */
					printf(".");
				else 
					printf("%c", temp[i_L]);
		}

			} /* For each of the 16 rows */
			
			printf("\n\t\tHit any key to continue, [ESC] key to return to menu: ");
			selection_L = getche();
			printf("\n");                   
		} while (selection_L !=0x1b); 
	}
	return( flerror_code_G );
} /* write_bin */
    
/******************************************************************/
/*      read_bin()                                                      */
/*                                                                */
/*      purpose:        This routine reads the device and outputs to the        */
/*              screen or to a file (in binary mode).             */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int read_bin( void )
{
	char selection_L;
	int i_L;
	int j_L;
	int deviceType = this_dev_GSP->device_no_GS;
	char temp[16];
	int end_card = 0;
	flerror_code_G = 0;
	
	fflush(stdin);
	printf("\nReading %s", this_dev_GSP->devname_GS);
	printf("\nEnter hexadecimal starting address: ");
	scanf("%Lx", &cardaddress);              
	printf("\n");   

	if (cardaddress >= max_addr_L) {
		flerror_code_G=ADDRESS_EXCEEDS_LIMITS;
		return(flerror_code_G);
	}               
	
	flerror_code_G=address_check();  
	/* send data to user screen, 256 bytes at a time */
	if (flerror_code_G==0){
		do {
			end_card = 0;
			for (j_L=0; j_L<16; j_L++) {
				/* For each of the 16 rows */
				ultoa(cardaddress, string_GA, 16);
				if (cardaddress==max_addr_L) 
				{
				   end_card = 1;
				   goto option;
				}
				
				printf("\n%8s: ", string_GA);

				/* Get the 16 bytes of the row */
				for ( i_L=0; i_L<16; i_L++ ) {
    /* moved this line here to */       flerror_code_G=address_check();  
    /* fix boundary errors     */       in_data_L = system_address;
    
    /* used to be below this line */    cardaddress++;   
					printf("%2.2x ", *in_data_L);                                   
					temp[i_L] = *in_data_L;
					if (temp[i_L] >= 0x07 && temp[i_L] <= 0x0d) 
						temp[i_L] = 0x2e;

				} /* for each of the 16 bytes in a row */
		
		/* Filter the output */
				for ( i_L = 0; i_L < 16; i_L++ ) {
					if ( temp[i_L] > 127) 
						/* filter system is on & is unprintable */
						printf(".");
					else 
						printf("%c", temp[i_L]);
			}

			} /* For each of the 16 rows */
			option: 
			
			if (end_card == 1) 
			{
			   printf ("\n\n\t\tEnd of card reached.");
			   printf("\n\t\tHit any key to return to menu: "); 
			}
			else printf("\n\t\tHit any key to continue, [ESC] key to return to menu: "); 
			selection_L = getche();
			printf("\n");                   
			if (end_card == 1) selection_L = 0x1b;
		} while (selection_L !=0x1b); 
	}
	return( flerror_code_G );
} /* read_bin */

/******************************************************************/
/*      program_cm_file()                                               */
/*                                                                */
/*      purpose:        This routine reads the device and outputs to the        */
/*              screen or to a file (in binary mode).             */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int program_cm_file( void )
{
	FILE *input;
	unsigned int selection_L;
	unsigned int selection_H;
	int eof_flag;
	
	if (!(input=fopen(filename, "rb")))     {
		printf("Error opening file %s.",filename);
		return(flerror_code_G);
	}       
	eof_flag=0;
	if (cardaddress&0x1)    {
		eof_flag=fscanf(input,"%c", &selection_L);
		selection_L=selection_L&0x00ff;
		write_data_B=selection_L;                                               
		flerror_code_G=program_byte();
		cardaddress++;
	}   
	while ((eof_flag!=EOF)&&(flerror_code_G==0)) 
	{
		eof_flag=fscanf(input,"%c", &selection_L);
		selection_L=selection_L&0x00ff;
		selection_H=0xff00;
		if (eof_flag != EOF) {
			eof_flag=fscanf(input,"%c", &selection_H);
			if (eof_flag != EOF) {
				selection_H=selection_H &0x00ff;
				selection_H=selection_H<<8;
				write_data=selection_L+selection_H;                                             
				flerror_code_G=program_word();
				cardaddress++;
				cardaddress++;
				if (cardaddress>max_addr_L+2)
					flerror_code_G=ADDRESS_EXCEEDS_LIMITS;
			}
			else {
				write_data_B=selection_L;                                               
				flerror_code_G=program_byte();
				cardaddress++;
				if (cardaddress>max_addr_L)
					flerror_code_G=ADDRESS_EXCEEDS_LIMITS;
			}                       
		}
	}                                                                               
	if (flerror_code_G==0) {        
		printf("\n\tWrite Passed. Hit any key to continue.");
		getche();
		printf("\n");                   
	}
	else {  
		printf("\n\tWrite Failed. Hit any key to continue.");
		getche();
		printf("\n");                   
	}
	fclose(input);
	return( flerror_code_G );
} /* program_cm_file*/

/******************************************************************/
/*      read_cm_file()                                                          */
/*                                                                */
/*      purpose:        This routine reads the device Common memory and                 */
/*           outputs to the to a file (in binary mode).           */
/*                                                                */
/******************************************************************/
/*      parameters:     none                                               */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int read_cm_file( void )                                                                                                                
{
	FILE *input;
	unsigned long int i;

	if (!(input=fopen(filename, "wb")))     {
		printf("Error opening file %s.",filename);
		return(flerror_code_G);
	}       
	for(i=0; i<numbytes;i++) {
		flerror_code_G=address_check();  
		in_data_L = system_address;
		fprintf(input,"%c",*in_data_L);                                 
		cardaddress++;
	}
	printf("\n\tRead to file done. Hit any key to continue.");
	getche();
	printf("\n");                   
	fclose(input);
	return( flerror_code_G );
} /* read_cm_file*/

/******************************************************************/
/*      program_word()                                                  */
/*                                                                      */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int program_word()
{       
	flerror_code_G = 0;
	flerror_code_G=address_check();
	in_data_L_W=system_address;
	if (*in_data_L_W!=0xffff)
		flerror_code_G=LOCATION_NOT_ERASED;
	if(flerror_code_G==0) {
		cmd_seq(WRITE);
		in_data_L_W=system_address;
		*in_data_L_W=write_data;
		flerror_code_G = dq7_poll(); /* poll by DQ7 */
		if (flerror_code_G != 0) {
			cmd_seq(RESET);
			return(flerror_code_G);
		}
		/* avoid re-erasing the sector that was just erased */
	} /* if valid sector */
	return( flerror_code_G );               
} /* successfully programmed word */

/******************************************************************/
/*      program_byte()                                                 */
/*                                                                */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int program_byte()
{       
	flerror_code_G = 0;
	flerror_code_G=address_check();
	in_data_L_B=system_address;
	odd_offset=0;
	if ((system_address & 0x1) !=0) 
		odd_offset=1;
	if (*in_data_L_B!=0xff)
		flerror_code_G=LOCATION_NOT_ERASED;
	if(flerror_code_G==0) {
		cmd_seq_byte(WRITE);
		in_data_L_B=system_address;
		*in_data_L_B=write_data_B;
		flerror_code_G = dq7_poll_byte(); /* poll by DQ7 */
	}
	if (flerror_code_G != 0)
		cmd_seq_byte(RESET);
	return( flerror_code_G );               
}

/******************************************************************/
/*      sector_erase()                                                 */
/*                                                                */
/*      purpose:        Performs the AMD sector Erase algorithm. This will      */
/*              only work with 5v 29F devices.                                        */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int sector_erase()
{       
	int deviceType = this_dev_GSP->device_no_GS;
	flerror_code_G = 0;
	write_data=0xffff;
	flerror_code_G=address_check();
	if (cardaddress >= max_addr_L) {
		flerror_code_G=ADDRESS_EXCEEDS_LIMITS;
		return(flerror_code_G);
	}               
	printf("\nErasing card : %s,  cardaddress: %lx", this_dev_GSP->devname_GS, cardaddress);
	if(flerror_code_G==0) {
		system_address = (system_address & ~(1));
		cmd_seq(RESET);
		cmd_seq(SECTOR_ERASE);
		/* According to datasheet, timeout needed here to account for 100us window */
		flerror_code_G = dq7_poll(); /* poll by DQ7 */
	}
	if (flerror_code_G != 0)
		cmd_seq(RESET);
	return(flerror_code_G);
} /* sector_erase */

/******************************************************************/
/*      sector_erase_sus()                                             */
/*                                                                */
/*      purpose:        Performs the AMD sector Erase algorithm. This will      */
/*              only work with 5v 29F devices which support erase */
/*              suspend.                                          */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int sector_erase_sus()
{       
	int deviceType = this_dev_GSP->device_no_GS;
	flerror_code_G = 0;
	write_data=0xffff;
	flerror_code_G=address_check();
	printf("\nErasing card : %s", this_dev_GSP->devname_GS);
	if(flerror_code_G==0)
	{
		cmd_seq(RESET);
		cmd_seq(SECTOR_ERASE);
		/* According to datasheet, timeout needed here to account for 100us window */
		cmd_seq(SECTOR_ERASE_SUSPEND);
	}
	if (flerror_code_G != 0)
		cmd_seq(RESET);
	return(flerror_code_G);
} /* sector_erase_sus */

/******************************************************************/
/*      sector_erase_res()                                             */
/*                                                                */
/*      purpose:        Performs the AMD sector Erase algorithm. This will      */
/*              only work with 5v 29F devices which support     */
/*              erase suspend.                                                                                  */
/*                                                                */
/******************************************************************/
/*      parameters:     int suspend = FALSE or TRUE, indicates whether to  */
/*                            use Erase Suspend or not            */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int sector_erase_res()
{       
	int deviceType = this_dev_GSP->device_no_GS;
	memctrl=CM_WRITE;
	flerror_code_G = 0;
	write_data=0xffff;
	flerror_code_G=address_check();
	printf("\nErasing card : %s", this_dev_GSP->devname_GS);
	if(flerror_code_G==0) {
		cmd_seq(SECTOR_ERASE_RESUME);
		/* According to datasheet, timeout needed here to account for 100us window */
		flerror_code_G = dq7_poll(); /* poll by DQ7 */
	}
	if (flerror_code_G != 0) 
		cmd_seq(RESET);
	return(flerror_code_G);
} /* sector_erase_resume */

/******************************************************************/
/*      card_erase()                                                    */
/*                                                                */
/*      purpose:        Performs the AMD sector Erase algorithm. This will      */
/*              only work with 5v 29F devices.                                  */
/*                                                                */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int card_erase()
{       
	int deviceType = this_dev_GSP->device_no_GS;
	flerror_code_G = 0;
	max_addr_L = this_dev_GSP->kb_size_GS;
	cardaddress=0;

	do {
		if (flerror_code_G==0) {
			flerror_code_G=sector_erase();
			cardaddress=cardaddress+(this_dev_GSP->sector_size);
		}                                                                                                                                                                
	} while ((cardaddress <= (max_addr_L - 1))&&(flerror_code_G==0));

	if ( flerror_code_G ) {
		cmd_seq(RESET);
		return(flerror_code_G);
	}               
	printf("\nSuccessfully erased %s \n", this_dev_GSP->devname_GS);
	return( flerror_code_G );               
} /* card_erase */

/******************************************************************/
/*      dq7_poll()                                                     */
/*                                                                */
/*      purpose:        Performs Data polling using DQ7 Toggle bit and DQ5      */
/*                              algorithm for program and erase operations. Error       */
/*                              recovery is minimal, that is, the user is warned        */
/*                              of the error, but the calling function is               */
/*                              responsible for exiting gracefully.                     */
/*                                                                                      */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int dq7_poll()
{
	unsigned int temp_L       = 0;
	unsigned int temp2_L      = 0;
	int deviceType            = this_dev_GSP->device_no_GS;

	flerror_code_G = 0;
	in_data_L_W=system_address;
	temp2_L = write_data;
	_bios_timeofday(_TIME_GETCLOCK, &oldcount);
	newcount=oldcount;
	
	while ((newcount-oldcount)<165) {        /* about 9 seconds     */
		/* read data */
		_bios_timeofday(_TIME_GETCLOCK, &newcount);
		in_data_word= *in_data_L_W;
		temp_L  = in_data_word;
	
		/* Check the DQ7 bit for DATA POLL */
		if ((temp2_L & 0x8080) == (temp_L & 0x8080)) {
			/* DQ7 = true data bit (programmed correctly) */
			/* read again for final data */
			in_data_word = *in_data_L_W;
				if ( write_data != in_data_word ) {
					/* if original data != in_data_word, even after DQ7 = true
					data, error */
					cmd_seq(RESET);
					flerror_code_G = DQ7_PROGRAM_FAILURE;
					return( flerror_code_G );
				}
				else 
				/* Embedded program worked properly as per DATA POLL */
				return( flerror_code_G );
		} /* DQ7 = true data bit (programmed correctly) */
	
		/* DQ7 still toggling */
		else {                          
			/* DQ7 = complement of true data (not finished programming yet) */
			if ((temp_L & 0x2020) == 0x2020) {
				/* DQ5 = 1; read again because maybe DQ7 changed 
				simultaneously with DQ5 */
				in_data_word = *in_data_L_W;
					/* Compare DQ7 with the original data */
					if ( (temp2_L & 0x8080) == (in_data_word & 0x8080) ) {
						/* We're done, the device passed the DATA POLL algorithm */
						return( flerror_code_G );
					}
					else {
					cmd_seq(RESET);
					flerror_code_G = DQ7_FAIL_AFTER_DQ5_2;
					return( flerror_code_G );
	
					} /* DQ7 is complement of true data, even after DQ5 = 1 */
			} /* DQ5 = '1' */
		} /* DQ7 !=write_data, try DQ5 */
	} /* for */
	flerror_code_G = DQ7_PROGRAM_FAILURE;
	return( flerror_code_G );
} /* dq7_poll */
/******************************************************************/
/*      dq7_poll_byte()                                                 */
/*                                                                      */
/*      purpose:        Performs Data polling using DQ7 Toggle bit and DQ5      */
/*                              algorithm for program and erase operations. Error       */
/*                              recovery is minimal, that is, the user is warned        */
/*                              of the error, but the calling function is               */
/*                              responsible for exiting gracefully.                     */
/*                                                                                      */
/******************************************************************/
/*  return value:   flerror_code = 0 : successful                 */
/*                               = anything : unsuccessful        */
/******************************************************************/    
int dq7_poll_byte()
{
	unsigned char temp_L_B       = 0;
	unsigned char temp2_L_B      = 0;
	int deviceType            = this_dev_GSP->device_no_GS;

	flerror_code_G = 0;
	in_data_L_B=system_address;
	temp2_L_B = write_data_B;
	
	for ( ; ; ) {
	/* read data */
	in_data_byte= *in_data_L_B;
	temp_L_B  = in_data_byte;

	/* Check the DQ7 bit for DATA POLL */
	if ((temp2_L_B & 0x80) == (temp_L_B & 0x80)) {
	
		/* DQ7 = true data bit (programmed correctly) */
		/* read again for final data */
	
		in_data_byte = *in_data_L_B;
		if ( write_data_B != in_data_byte ) {
		/* if original data != in_data_word, even after DQ7 = true
			data, error */
			cmd_seq(RESET);
			flerror_code_G = DQ7_PROGRAM_FAILURE;
			return( flerror_code_G );
			}
		else 
		/* Embedded program worked properly as per DATA POLL */
			return( flerror_code_G );
	
		} /* DQ7 = true data bit (programmed correctly) */
	
	/* DQ7 still toggling */
	else {                          
		/* DQ7 = complement of true data (not finished programming yet) */
		if ((temp_L_B & 0x20) == 0x20) {
			/* DQ5 = 1; read again because maybe DQ7 changed 
			simultaneously with DQ5 */
			in_data_byte = *in_data_L_B;
	
			/* Compare DQ7 with the original data */
			if ( (temp2_L_B & 0x80) == (in_data_byte & 0x80) ) {
				/* We're done, the device passed the DATA POLL algorithm */
				return( flerror_code_G );
				}
			else {
				cmd_seq(RESET);
				flerror_code_G = DQ7_FAIL_AFTER_DQ5_2;
				return( flerror_code_G );
	
				} /* DQ7 is complement of true data, even after DQ5 = 1 */
			} /* DQ5 = '1' */
		} /* DQ7 !=write_data, try DQ5 */
	} /* for */
} /* dq7_poll */


/******************************************************************/
/*      Cmd_seq()                                                                       */
/*                                                                */
/*      purpose:        Performs the Operation setup,                                                           */
/******************************************************************/
/*  return value:                                                 */
/******************************************************************/
void cmd_seq(int unlock_operation)              
{
	switch (unlock_operation) {
		case RESET:                             
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xf0f0;
			break;
		case AUTOSELECT:                                
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0x9090;
			break;
		case WRITE:                             
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xa0a0;
			break;
		case SEGMENT_ERASE:
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0x8080;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0x1010;
			break;
		case SECTOR_ERASE:
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0x8080;
			in_data_L_W=system_base_address+0xaaa;
			*in_data_L_W=0xaaaa;
			in_data_L_W=system_base_address+0x554;
			*in_data_L_W=0x5555;
			in_data_L_W=system_address;       
			*in_data_L_W=0x3030;
			break;
		case SECTOR_ERASE_SUSPEND:
			in_data_L_W=system_address;
			*in_data_L_W=0xB0B0;
			break;          
		case SECTOR_ERASE_RESUME:
			in_data_L_W=system_address;
			*in_data_L_W=0x3030;
			break;

		default:
			break;
	}                                               
	in_data_L_W=system_address;
} /* unlock_sequence */

/******************************************************************/
/*      Cmd_seq_byte()                                                  */
/*                                                                */
/*      purpose:        Performs the Operation setup,                                                           */
/******************************************************************/
/*  return value:                                                 */
/******************************************************************/
void cmd_seq_byte(int unlock_operation1)                
{
	switch (unlock_operation1) {
		case RESET:                             
			in_data_L_B=system_base_address+0xaaaa+odd_offset;
			*in_data_L_B=0xaa;
			in_data_L_B=system_base_address+0x5554+odd_offset;
			*in_data_L_B=0x55;
			in_data_L_B=system_base_address+0xaaaa+odd_offset;
			*in_data_L_B=0xf0;
			break;
		case WRITE:                             
			in_data_L_B=system_base_address+0xaaaa+odd_offset;
			*in_data_L_B=0xaa;
			in_data_L_B=system_base_address+0x5554+odd_offset;
			*in_data_L_B=0x55;
			in_data_L_B=system_base_address+0xaaaa+odd_offset;
			*in_data_L_B=0xa0;
			break;
		default:
			break;
	}                                               
	in_data_L_B=system_address;
} /* unlock_sequence */



/******************************************************************/
/*      address_check()                                                 */
/*                                                                */
/*      purpose:                                                                                                                                                */
/******************************************************************/
/*  return value: check_error                                     */
/******************************************************************/
int address_check()
{
	int check_error=0;
	if ((cardaddress>max_cardaddress) || (cardaddress<min_cardaddress)) {
		cardoffset=(cardaddress/window_size)*window_size;
		cardoffset_DI=((cardoffset>>12) & (0x03fff));
		min_cardaddress=cardoffset;
		max_cardaddress=cardoffset+window_size-1;
		check_error=set_page(cardoffset_DI,memctrl);
	}
	system_address=system_base_address+(cardaddress-cardoffset);
	return(check_error);
} /* address_check() */

/******************************************************************/
/*      autoselect_seq()                                               */
/*                                                                */
/*      purpose:        Performs the Auto select setup, to                                      */
/*                              obtain the manufacturer and device IDs.                                 */
/*                                                                */
/******************************************************************/
/*  return value: none                                            */
/******************************************************************/
void autoselect_seq(int devtype_L)
{
	cardaddress=0;
	address_check();
	cmd_seq(AUTOSELECT);
	in_data_L_W=system_base_address;
	/* Obtain the manufacturer code */
	m_code_G = *in_data_L_W;
	/* Obtain the device code */
	in_data_L_W=system_base_address+2;
	d_code_G = *(in_data_L_W);
	cmd_seq(RESET); 
} /* autoselect_seq */


