/************************************************************************/
/*                                                                      */
/*  AMD Embedded Flash Memory Card Evaluation Kit                       */
/*  File name: socket.c                                                 */
/*  Revision:  1.4 Beta 6/25/96                                             */
/*                                                                      */
/* Copyright (c) 1995 ADVANCED MICRO DEVICES, INC. All Rights Reserved. */
/* This software is unpublished and contains the trade secrets and      */
/* confidential proprietary information of AMD. Unless otherwise        */
/* provided in the Software Agreement associated herewith, it is        */
/* licensed in confidence "AS IS" and is not to be reproduced in whole  */
/* or part by any means except for backup. Use, duplication, or         */
/* disclosure by the Government is subject to the restrictions in       */
/* paragraph (b) (3) (B) of the Rights in Technical Data and Computer   */
/* Software clause in DFAR 52.227-7013 (a) (Oct 1988).                  */
/* Software owned by                                                    */
/* Advanced Micro Devices, Inc.,                                        */
/* One AMD Place,                                                       */
/* P.O. Box 3453                                                        */
/* Sunnyvale, CA 94088-3453.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Revision History                                                     */
/*                                                                      */
/* Rev.  Date        By    Comments                                     */
/* 1.4   6/25/96     KT    Socket related files socket.c,socket.h in C. */
/* 1.8B  10/10/96    KD    Adding in option for socket 0 or 1.          */            
/*                                                                      */
/*                                                                      */
/************************************************************************/

#include "socket.h"

#define INDEX_REG_PORT 0x03e0   /*Index register port address*/
#define DATA_REG_PORT 0x03e1    /*Data register port address*/

void write_reg(unsigned char regadd,unsigned char regdata);
int read_reg(unsigned char regadd);
int find_adapter();
void init_socket();
void save_regs();
void restore_regs();
int set_window();
int set_page();
void disp_reg();

/*****  Globals **************/
extern unsigned char slotoff; /*  first slot default   */
struct pcic_reg_data saved_regs;

int find_adapter()
{
	unsigned char regadd;
	unsigned char regdata;

	regadd = REVID;
	regdata=read_reg(regadd);

	printf("Revision ID = %x\n", regdata);

	if ((regdata == ID1)||(regdata==ID2)||(regdata==ID)){
		save_regs();
		init_socket();
		return(0);
	}
	else
		return(ERROR_G);
}

void write_reg(unsigned char regadd,unsigned char regdata)
{
	outp(INDEX_REG_PORT,regadd);
	outp(DATA_REG_PORT,regdata);
}/* end write to PCIC register */

int read_reg(unsigned char regadd)
{
	unsigned char regdata;

   outp(INDEX_REG_PORT,regadd);
   regdata=inp(DATA_REG_PORT);
   return(regdata);
}/* end read to PCIC register */

void init_socket()
{
	unsigned char regadd;
	unsigned char regdata;

	regadd = slotoff + POWER;
	regdata = 0xBA;  
	write_reg(regadd,regdata);
	regadd = slotoff + INT_CTRL;
	regdata = 0x40;
	write_reg(regadd,regdata);
	regadd = slotoff + DELTA_STAT;
	regdata = 0x0;
	write_reg(regadd,regdata);
	regadd = slotoff + INT_CNFG;
	regdata = 0x0;
	write_reg(regadd,regdata);
	regadd = slotoff + MAP_EN;
	regdata = 0x21;
	write_reg(regadd,regdata);
	regadd = slotoff + IO_CTRL;
	regdata = 0x0;
	write_reg(regadd,regdata);
}

void save_regs()
{
	unsigned char regadd;

	regadd = REVID;
	saved_regs.REVID_data = read_reg(regadd);

	regadd = slotoff + STATUS;
	saved_regs.STATUS_data = read_reg(regadd);

	regadd = slotoff + POWER;
	saved_regs.POWER_data = read_reg(regadd);

	regadd = slotoff + INT_CTRL;
	saved_regs.INT_CTRL_data = read_reg(regadd);

	regadd = slotoff + DELTA_STAT;
	saved_regs.DELTA_STAT_data = read_reg(regadd);

	regadd = slotoff + INT_CNFG;
	saved_regs.INT_CNFG_data = read_reg(regadd);

	regadd = slotoff + MAP_EN;
	saved_regs.MAP_EN_data = read_reg(regadd);

	regadd = slotoff + IO_CTRL;
	saved_regs.IO_CTRL_data = read_reg(regadd);

	regadd = slotoff + SMA0sl;
	saved_regs.SMA0sl_data = read_reg(regadd);

	regadd = slotoff + SMA0sh;
	saved_regs.SMA0sh_data = read_reg(regadd);

	regadd = slotoff + SMA0el;
	saved_regs.SMA0el_data = read_reg(regadd);

	regadd = slotoff + SMA0eh;
	saved_regs.SMA0eh_data = read_reg(regadd);

	regadd = slotoff + CMA0l;
	saved_regs.CMA0l_data = read_reg(regadd);

	regadd = slotoff + CMA0h;
	saved_regs.CMA0h_data = read_reg(regadd);

	regadd = slotoff + Gen_Ctrl;
	saved_regs.Gen_Ctrl_data = read_reg(regadd);

	regadd = Glob_Ctrl;
	saved_regs.Glob_Ctrl_data = read_reg(regadd);

	regadd = Volt_Sense;
	saved_regs.Volt_Sense_data = read_reg(regadd);

	regadd = slotoff + Volt_Select;
	saved_regs.Volt_Select_data = read_reg(regadd);

}
		
void restore_regs()
{
	unsigned char regadd;

	regadd = REVID;
	write_reg(regadd,saved_regs.REVID_data);

	regadd = slotoff + STATUS;
	write_reg(regadd,saved_regs.STATUS_data);

	regadd = slotoff + POWER;
	write_reg(regadd,saved_regs.POWER_data);

	regadd = slotoff + INT_CTRL;
	write_reg(regadd,saved_regs.INT_CTRL_data);

	regadd = slotoff + DELTA_STAT;
	write_reg(regadd,saved_regs.DELTA_STAT_data);

	regadd = slotoff + INT_CNFG;
	write_reg(regadd,saved_regs.INT_CNFG_data);

	regadd = slotoff + MAP_EN;
	write_reg(regadd,saved_regs.MAP_EN_data);

	regadd = slotoff + IO_CTRL;
	write_reg(regadd,saved_regs.IO_CTRL_data);

	regadd = slotoff + SMA0sl;
	write_reg(regadd,saved_regs.SMA0sl_data);

	regadd = slotoff + SMA0sh;
	write_reg(regadd,saved_regs.SMA0sh_data);

	regadd = slotoff + SMA0el;
	write_reg(regadd,saved_regs.SMA0el_data);

	regadd = slotoff + SMA0eh;
	write_reg(regadd,saved_regs.SMA0eh_data);

	regadd = slotoff + CMA0l;
	write_reg(regadd,saved_regs.CMA0l_data);

	regadd = slotoff + CMA0h;
	write_reg(regadd,saved_regs.CMA0h_data);

	regadd = slotoff + Gen_Ctrl;
	write_reg(regadd,saved_regs.Gen_Ctrl_data);

	regadd = Glob_Ctrl;
	write_reg(regadd,saved_regs.Glob_Ctrl_data);

	regadd = Volt_Sense;
	write_reg(regadd,saved_regs.Volt_Sense_data);

	regadd = slotoff + Volt_Select;
	write_reg(regadd,saved_regs.Volt_Select_data);

}
	
int set_page(int cardoffset, unsigned char page_cnfg)
{
	unsigned char regadd;
	unsigned char regdata;

	unsigned char win_addr_regl, win_addr_regh;
   int win_addr_reg;
	unsigned char card_addr_regl, card_addr_regh;
   int card_addr_reg;
_asm int 3
	regadd = slotoff + SMA0sl;
	win_addr_regl=read_reg(regadd);

	regadd = slotoff + SMA0sh;
	win_addr_regh=read_reg(regadd);

	win_addr_reg = ((((unsigned int)(win_addr_regh))<<8)|((unsigned int)(win_addr_regl))) & 0x0fff;
	card_addr_reg = 0x3fff-(0x3fff&(win_addr_reg - cardoffset))+1;

/* is it attribute memory? */
	if (0x1 & page_cnfg) 
		card_addr_regh =(unsigned char)((card_addr_reg>>8)|0x40); /* Yes */
	else
		card_addr_regh =(unsigned char)((card_addr_reg>>8)&0xbf);
/* is it write protect? */
//      if (0x4 & page_cnfg) 
//              card_addr_regh = card_addr_regh|0x80; /* Yes */
//      else
//              card_addr_regh = card_addr_regh&0x7f;

	card_addr_regl=(unsigned char)(card_addr_reg);

	regadd = slotoff + CMA0l;
	write_reg(regadd,card_addr_regl);

	regadd = slotoff + CMA0h;
	write_reg(regadd,card_addr_regh);
	return(0);
}

int set_window(unsigned int window_base, unsigned int window_cnfg)
{
	unsigned char regadd;
	unsigned char regdata;
	unsigned char window_size=0xf; /* (window_size+1)*4096 bytes */
											 /* 64k in this case    */      
	unsigned int win_addr_reg;
	
/* disable window   */
	regadd = slotoff + MAP_EN;
	regdata= 0x20;
	write_reg(regadd,regdata);

	regadd = slotoff + SMA0sl;
	regdata= (char)window_base;
	write_reg(regadd,regdata);

	regadd = slotoff + SMA0sh;
	regdata= (char)(window_base>>8);
	if (window_cnfg & 0x0002==0x0002)
		regdata=regdata|0x80;   /* if 16-bit access */
	else regdata=regdata&0x7f;  /* if 8-bit access  */
	write_reg(regadd,regdata);

/*system memory address lower stop byte */
	regadd = slotoff + SMA0el;
   regdata = (char)window_base + window_size;
	write_reg(regadd,regdata);

/*set timing requirements as well as system memory address upper stop byte */
/* d7=wait state bit 1
	d6=wait state bit 0 */

	regadd = slotoff + SMA0eh;
	regdata = ((char)(window_base>>8))|(0xc0 & ((char)(window_cnfg<<8)));
	write_reg(regadd,regdata);

/* enable window    */
	regadd = slotoff + MAP_EN;
	regdata= 0x21;
	write_reg(regadd,regdata);
	
	return(0);
 }




















  
