/************************************************************************/
/*                                                                      */
/*  AMD Embedded Flash Memory Card Evaluation Kit                       */
/*  File name: socket.h                                                 */
/*  Revision:  1.4 Beta 6/25/96                                             */
/*                                                                      */
/* Copyright (c) 1995 ADVANCED MICRO DEVICES, INC. All Rights Reserved. */
/* This software is unpublished and contains the trade secrets and      */
/* confidential proprietary information of AMD. Unless otherwise        */
/* provided in the Software Agreement associated herewith, it is        */
/* licensed in confidence "AS IS" and is not to be reproduced in whole  */
/* or part by any means except for backup. Use, duplication, or         */
/* disclosure by the Government is subject to the restrictions in       */
/* paragraph (b) (3) (B) of the Rights in Technical Data and Computer   */
/* Software clause in DFAR 52.227-7013 (a) (Oct 1988).                  */
/* Software owned by                                                    */
/* Advanced Micro Devices, Inc.,                                        */
/* One AMD Place,                                                       */
/* P.O. Box 3453                                                        */
/* Sunnyvale, CA 94088-3453.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Revision History                                                     */
/*                                                                      */
/* Rev.  Date        By    Comments                                     */
/* 1.4   6/25/96     KT    Socket related files socket.c,socket.h in C. */
/* 1.8B  10/7/96     KD    Changed slot0 registers to slot 1 registers. */
/*       10/10/96    KD    Added option for slot0 or slot 1.            */
/************************************************************************/

#define ID1      0x83 /* register 0 value for VG-468 */
#define ID2      0x82 /* register 0 value for VG-468 */
#define ID       0x84 /* register 0 value for VG-469 */

/*---------------------------------------------------------------------
   These are register definitions.
---------------------------------------------------------------------*/

#define REVID          0    /* ID and Revision register                         */
#define STATUS         0x1    /* Status of specific card socket         */
#define POWER          0x2    /* Power and reset drive control          */
#define INT_CTRL       0x3    /* Interrupt and general control          */
#define DELTA_STAT     0x4    /* Card status change (Delta STATus)      */
#define INT_CNFG       0x5    /* Status INT config                      */
#define MAP_EN         0x6    /* Address Window enable                  */
#define IO_CTRL        0x7    /* I/O control                            */

#define IOA0sl         0x8  /* I/O Address 0 start Low Byte             */
#define IOA0sh         0x9  /* I/O Address 0 start High Byte            */
#define IOA0el         0x0A /* I/O Address 0 stop Low Byte              */
#define IOA0eh         0x0B /* I/O Address 0 stop High Byte             */

#define IOA1sl         0x0C /* I/O Address 1 start Low Byte             */
#define IOA1sh         0x0D /* I/O Address 1 start High Byte            */
#define IOA1el         0x0E /* I/O Address 1 stop Low Byte              */
#define IOA1eh         0x0F /* I/O Address 1 stop High Byte             */

#define SMA0sl         0x10 /* System Memory Address 0 start Low byte   */
#define SMA0sh         0x11 /* System Memory Address 0 start High byte  */
#define SMA0el         0x12 /* System Memory Address 0 stop Low byte    */
#define SMA0eh         0x13 /* System Memory Address 0 stop High byte   */
#define CMA0l          0x14 /* Card Memory Offset Address 0 Low byte    */
#define CMA0h          0x15 /* Card Memory Offset Address 0 Low byte    */

#define Gen_Ctrl       0x16 /* Card detect and general control register */

#define SMA1sl         0x18 /* System Memory Address 1 start Low byte   */
#define SMA1sh         0x19 /* System Memory Address 1 start High byte  */
#define SMA1el         0x1A /* System Memory Address 1 stop Low byte    */
#define SMA1eh         0x1B /* System Memory Address 1 stop High byte   */
#define CMOA1l         0x1C /* Card Memory Offset Address 1 Low byte    */
#define CMOA1h         0x1D /* Card Memory Offset Address 1 High byte   */

#define Glob_Ctrl      0x1E /* Global Control register                  */
								  
#define Volt_Sense     0x1F /* Voltage Sense register                   */

#define SMA2sl         0x10 /* System Memory Address 2 start Low byte   */
#define SMA2sh         0x11 /* System Memory Address 2 start High byte  */
#define SMA2el         0x12 /* System Memory Address 2 stop Low byte    */
#define SMA2eh         0x13 /* System Memory Address 2 stop High byte   */
#define CMOA2l         0x14 /* Card Memory Offset Address 2 Low byte    */
#define CMOA2h         0x15 /* Card Memory Offset Address 2 High byte   */

#define SMA3sl         0x18 /* System Memory Address 3 start Low byte   */
#define SMA3sh         0x19 /* System Memory Address 3 start High byte  */
#define SMA3el         0x1A /* System Memory Address 3 stop Low byte    */
#define SMA3eh         0x1B /* System Memory Address 3 stop High byte   */
#define CMOA3l         0x1C /* Card Memory Offset Address 3 Low byte    */
#define CMOA3h         0x1D /* Card Memory Offset Address 3 High byte   */
#define Volt_Select    0x2F /* Voltage Select register                  */
								  
#define SMA4sl         0x30 /* System Memory Address 4 start Low byte   */
#define SMA4sh         0x31 /* System Memory Address 4 start High byte  */
#define SMA4el         0x32 /* System Memory Address 4 stop Low byte    */
#define SMA4eh         0x33 /* System Memory Address 4 stop High byte   */
#define CMOA4l         0x34 /* Card Memory Offset Address 4 Low byte      */
#define CMOA4h         0x35 /* Card Memory Offset Address 4 High byte  */

struct pcic_reg_data {                                                                                                                            
	unsigned char REVID_data;
	unsigned char STATUS_data;
	unsigned char POWER_data;
	unsigned char INT_CTRL_data;
	unsigned char DELTA_STAT_data;
	unsigned char INT_CNFG_data;
	unsigned char MAP_EN_data;
	unsigned char IO_CTRL_data;

	unsigned char IOA0sl_data;
	unsigned char IOA0sh_data;
	unsigned char IOA0el_data;
	unsigned char IOA0eh_data;

	unsigned char IOA1sl_data;
	unsigned char IOA1sh_data;
	unsigned char IOA1el_data;
	unsigned char IOA1eh_data;

	unsigned char SMA0sl_data;
	unsigned char SMA0sh_data;
	unsigned char SMA0el_data;
	unsigned char SMA0eh_data;
	unsigned char CMA0l_data;
	unsigned char CMA0h_data;

	unsigned char Gen_Ctrl_data;

	unsigned char SMA1sl_data;
	unsigned char SMA1sh_data;
	unsigned char SMA1el_data;
	unsigned char SMA1eh_data;
	unsigned char CMOA1l_data;
	unsigned char CMOA1h_data;

	unsigned char Glob_Ctrl_data;
								  
	unsigned char Volt_Sense_data;

	unsigned char SMA2sl_data;
	unsigned char SMA2sh_data;
	unsigned char SMA2el_data;
	unsigned char SMA2eh_data;
	unsigned char CMOA2l_data;
	unsigned char CMOA2h_data;

	unsigned char SMA3sl_data;
	unsigned char SMA3sh_data;
	unsigned char SMA3el_data;
	unsigned char SMA3eh_data;
	unsigned char CMOA3l_data;
	unsigned char CMOA3h_data;

	unsigned char Volt_Select_data;
								  
	unsigned char SMA4sl_data;
	unsigned char SMA4sh_data;
	unsigned char SMA4el_data;
	unsigned char SMA4eh_data;
	unsigned char CMOA4l_data;
	unsigned char CMOA4h_data;
};


/***************** Global Defines *************/
#define ERROR_G                 0xff

/* set_page */
unsigned short int memctrl;

/* set_window */
int window_base;
int window_dx;
unsigned short int window_dh;
unsigned short int window_dl;
